<?php

namespace Bixo\Profile\Http\Controllers;

use Exception;
use Bixo\Profile\Actions\ProfileWorkflow;
use Bixo\Profile\Http\Requests\ProfileWorkflowRequest;
use Bixo\Profile\Http\Resources\ProfileResource;
use Bixo\Profile\Models\Profile;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class ProfileWorkflowController extends BaseController
{
    /**
     * Action controller function for profile.
     *
     * @param Model $profile
     * @param action next action for the profile.
     *
     * @return Response
     */
    public function __invoke(ProfileWorkflowRequest $request, Profile $profile, $trasition)
    {
        try {
            $request = $request->all();
            $profile = ProfileWorkflow::run($trasition, $profile, $request);
            $data = new ProfileResource($profile);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('profile::profile.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the profile and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $profile
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $profile, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $profile = Profile::findBySignedId($profile);
        $transitions = $this->transitions($profile, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('profile::profile.name') . ' ' . trans('Approvals'))
            ->view('profile::public.profile.workflow')
            ->layout('mobile')
            ->data(compact('profile', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the profile.
     *
     * @param PublicRequest $request
     * @param string $profile
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $profile, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $profile = Profile::findBySignedId($profile);
        $user = user();
        $profile = ProfileWorkflow::run($transition, $profile, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($profile, $user)
    {
        $transitions = [];
        foreach ($profile->workflow()->transitions($profile) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.profile.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'profile' => $profile->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $profile->workflow()->form($value);
            $array['label'] = trans('profile::profile.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
